////////////////////////////////////////////////////////////////////////////////
//
//  THIS CODE IS NOT REDISTRIBUTABLE OR APPROVED FOR USE IN/ON ANY OTHER UI 
//  ELEMENT OR PRODUCT COMPONENT.
//  Copyright (c) Microsoft Corporation.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
var outlookApplication;
var wshShell; 
var waitSpinner;


var DEBUG = false;
var outlookFailed = false;
var outlookInvalid = false;
var propShowOrganizer = false;
var propShowPrivateAppts = false;
var propGadgetSize = "normal";
var themeName = "blue";
var appointmentsToCache = 20;
var maxAppointmentsToScan = 50;
var useTransitions = false;
var isRefreshing = false;

var REFRESH_UI_RATE = 60000;		// 1 Minute
var REFRESH_DATA_RATE = 300000;		// 5 Minutes
var REFRESH_NO_OUTLOOK = 30000;		// 0.5 Minutes
var REFRESH_START_OUTLOOK = 20000;   // 20 seconds

var lastUpdateDate = new Date();
var appointments = new Object();
var numAppointments = 0;

var dataRefreshTimer;
var gadgetRefreshTimer;
var firstBoot = true;
var weekDayList = GetWeekDayNames();
var dateFormatter = new CustomDateFormatter(weekDayList);
//var STR_WeekDayNames = L_WeekDayNamesList_Text.split(",");

////////////////////////////////////////////////////////////////////////////////
//
// Initialize
//
////////////////////////////////////////////////////////////////////////////////
window.onerror=handleErr;
System.Gadget.settingsUI = "Settings.html";
System.Gadget.onSettingsClosed = settingsClosed;
System.Gadget.onShowSettings = settingsOpen;
System.Gadget.onDock = gadgetDocking;
System.Gadget.onUndock = gadgetDocking;
System.Gadget.visibilityChanged = gadgetVisibilityChange;
wshShell = new ActiveXObject("WScript.Shell");
dbgOut("ApptGadget: " + new Date().toString());

////////////////////////////////////////////////////////////////////////////////
//
// AppointmentData Object
//
////////////////////////////////////////////////////////////////////////////////
function AppointmentData() 
{
    this.Subject = "";
    this.Organizer = "";
    this.DateString = "";
    this.EntryID = "";
    this.Location = "";
    this.Valid = false;
    this.Colors = new String();
    this.BusyState = 0;
    this.IsPrivate = false;
    this.IsRecurring = false;
    this.InvalidAt = new Date();
    this.IsConflict = false;
}

////////////////////////////////////////////////////////////////////////////////
//
// Debug: Write error messages to debug logging
//
////////////////////////////////////////////////////////////////////////////////
function handleErr(msg,url,l)
{
    dbgOut("Problem: " + msg + "\nLine: " + l);
    return true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Setup gadget state and refresh data from Outlook
//
////////////////////////////////////////////////////////////////////////////////
function init() 
{
    if (!waitSpinner)
    {
        waitSpinner = new getSpinner("divWaitForData");
    }
    waitSpinner.hide();
    waitSpinner.stop();        

    loadSettings();

    if (firstBoot == true)	
    {
        dbgOut("First boot");
        var result = CheckOutlookVersion();
        if (result == false)
        {
            outlookInvalid = true;
            showInvalidOutlook();
            return;
        }
        
        showRefreshingData();
        setDataRefreshTimer(1000);
        firstBoot = false;
    } 
    else 
    {
        showRefreshingData();
        clearDataRefreshTimer();
        setDataRefreshTimer(1000);
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Transition gadget between docked and undocked state
//
////////////////////////////////////////////////////////////////////////////////
function gadgetDocking()
{
    initGadgetSize();
    if (!outlookInvalid)
    {
        updateDisplay();
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Handle the onkeydown event from the body
//
////////////////////////////////////////////////////////////////////////////////
function bodyKeyDown()
{
    if (event.ctrlKey == true && event.keyCode == 116)
    {
        updateDisplay();
    }
    else if (event.keyCode == 116)
    {
        init();
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Force an update to occur when the gadget becomes visible again
//
////////////////////////////////////////////////////////////////////////////////
function gadgetVisibilityChange()
{
    if (firstBoot == true)
    {
        return;
    }
    
    if (System.Gadget.visible)
    {
        dbgOut("Gadget is now visible...");
        // Check to see if the last update was too long ago
        var currentDate = new Date();
        if (currentDate.setMinutes(currentDate.getMinutes() - 5) > lastUpdateDate)
        {
            dbgOut("Forcing refresh because data is stale.");
            refreshAppointments();
        }
    }
    else
    {
        dbgOut("Gadget is no longer visible");
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Display error message
//
////////////////////////////////////////////////////////////////////////////////
function showInvalidOutlook()
{
    setNoData(true, L_WrongOlkVersion_ErrorMessage, false);
}

////////////////////////////////////////////////////////////////////////////////
//
// Launch the outlook process
//
////////////////////////////////////////////////////////////////////////////////
function launchOutlook()
{
    try
    {
        System.Shell.execute("outlook.exe", "/recycle");
        showRefreshingData();
        clearDataRefreshTimer();
        setDataRefreshTimer(REFRESH_START_OUTLOOK);	    
    }
    catch (err)
    {
        
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Clear gadget display and show refreshing message
//
////////////////////////////////////////////////////////////////////////////////
function showRefreshingData()
{
    waitSpinner.show();
    waitSpinner.start();
    setNoData(true, L_RefreshingData_Message);
    isRefreshing = true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Displays a message to user in place of showing data
//
////////////////////////////////////////////////////////////////////////////////
function setNoData(visible, message, showLink)
{
    dbgOut("SetNoData: " + visible + ";" + message + ";" + showLink);
    if (visible)
    {
        divDataArea.style.display = "none";
        divNoDataMessage.innerText = message;
        divNoData.style.display = "";
        if (showLink == true)
        {
            divNoDataLink.style.display = "";
            linkLaunchOutlook.innerText = L_LaunchOutlookLink_Text;
        }
        else
        {
            divNoDataLink.style.display = "none";
        }
    }
    else
    {
        divNoData.style.display = "none";
        divDataArea.style.display = "";
        waitSpinner.stop();
        waitSpinner.hide();
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Reset the timeout that drives collecting new data
//
////////////////////////////////////////////////////////////////////////////////
function clearDataRefreshTimer()
{
    window.clearTimeout(dataRefreshTimer);
}

////////////////////////////////////////////////////////////////////////////////
//
// Set the timeout that drives collecting new data to a particular value
//
////////////////////////////////////////////////////////////////////////////////
function setDataRefreshTimer(value)
{
    dataRefreshTimer = window.setTimeout("refreshAppointments()", value);
}

////////////////////////////////////////////////////////////////////////////////
//
// Set state for the opening the settings window
//
////////////////////////////////////////////////////////////////////////////////
function settingsOpen(evt)
{
    clearDataRefreshTimer();
}

////////////////////////////////////////////////////////////////////////////////
//
// Set state after the settings window is closed
//
////////////////////////////////////////////////////////////////////////////////
function settingsClosed(evt) 
{ 
    if (evt.closeAction == evt.Action.commit) 
    {
        init();
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Load persisted settings for the gadget
//
////////////////////////////////////////////////////////////////////////////////
function loadSettings() 
{
    propShowOrganizer = System.Gadget.Settings.read("showOrganizer");
    propShowPrivateAppts = System.Gadget.Settings.read("showPrivateAppts");
    if (propShowPrivateAppts != "true")
    {
        propShowPrivateAppts = "false";
    }
    propGadgetSize = System.Gadget.Settings.read("gadgetSize");
    if ("large" != propGadgetSize)
    {
        propGadgetSize = "normal";
    }
    
    // Load the Office theme
    var theme = 0;
    try
    {
        theme = wshShell.RegRead("HKCU\\Software\\Microsoft\\Office\\12.0\\Common\\Theme");
    }
    catch (err)
    {
        dbgOut("Problem reading office theme: " + err.description);
    }
    
    applyTheme(theme);
}

////////////////////////////////////////////////////////////////////////////////
//
// Update the display graphics to use a particular theme
//
////////////////////////////////////////////////////////////////////////////////
function applyTheme(themeId)
{
    var themeNumber = themeId;
    switch(Number(themeNumber))
    {
        case 3:	// Black
        case 2: // Silver
            themeName = "gray";
            break;
        default: // Blue
            themeName = "blue";
            break;
    }
    initGadgetSize();
}

////////////////////////////////////////////////////////////////////////////////
//
// Change the size of the gadget to be consistent with settings
//
////////////////////////////////////////////////////////////////////////////////
function initGadgetSize()
{
    if (System.Gadget.docked)
    {
        document.body.style.width = "130px";
        document.styleSheets.item(1).href = "css/docked.css";			
        GadgetBody.style.background = "url(gimage:///images/bk_" + propGadgetSize + "_" + themeName + ".png)";
    }
    else
    {
        document.body.style.width = "180px";
        document.styleSheets.item(1).href = "css/undocked.css";			
        GadgetBody.style.background = "url(gimage:///images/bk_" + propGadgetSize + "_wide_" + themeName + ".png)";
    }

    switch (propGadgetSize)
    {
        case "large":
            document.styleSheets.item(0).href = "css/appointments_large.css";
            document.body.style.height = "279px";
            break;
        
        default:
            document.styleSheets.item(0).href = "css/appointments_normal.css";			
            document.body.style.height = "179px";
            break;
    }
    

}

////////////////////////////////////////////////////////////////////////////////
//
// Connect to Outlook and retrieve upcoming appointments
// UsesOutlook: True
//
////////////////////////////////////////////////////////////////////////////////
function refreshAppointments()
{
    clearDataRefreshTimer();

    // Skip doing a data update if the gadget isn't visible
    if (System.Gadget.visible == false)
    {
        setDataRefreshTimer(REFRESH_DATA_RATE);
        return;
    }

    dateFormatter.reload();
    isRefreshing = false;

    var connected = ConnectToOutlook();
    if (connected == false)
    {
        updateDisplay();
        setDataRefreshTimer(REFRESH_NO_OUTLOOK);
        return;
    }

    var application = outlookApplication;
    if (isNull(application))
    {
        setDataRefreshTimer(REFRESH_NO_OUTLOOK);
        application = null;
        DisconnectFromOutlook();
        CollectGarbage();
        return;
    }

    try
    {	
        // Reset the appointment cache
        appointments = new Object();
        var date = new Date();
        //var nowDate = formatDate(date);
        var nowDate = dateFormatter.toOutlookFilterString(date);
        
        var filterCurrent = "([Start] < '" + nowDate + "') AND ([End] > '" + nowDate + "')";
        var filterFuture = "([Start] >= '" + nowDate + "')";
        dbgOut("current:" + filterCurrent);
        dbgOut("future: " + filterFuture);
        
        var session = application.Session;
        var folder = session.GetDefaultFolder(9);
        var items = folder.Items;
        items.Sort("[Start]");
        items.IncludeRecurrences = true;
    
        // Look for appointments occuring right now
        var currentCalItems = items.Restrict(filterCurrent);
        
        var itemCount = 0;
        for(var i=1; i<=maxAppointmentsToScan && i<currentCalItems.Count && itemCount<appointmentsToCache; i++) 
        {
            var item = currentCalItems.Item(i);
            if (isNull(item))
            {
                break;
            }
            
            var cachedAppointment = processAppointment(item);
            if (cachedAppointment != null)
            {
                itemCount++;
                appointments[itemCount] = cachedAppointment;
            }
            
            item = null;
        }
        currentCalItems = null;
        
        // Look for future appointments
        var futureCalItems = items.Restrict(filterFuture);
    
        for(var i=1; i<=maxAppointmentsToScan && i<futureCalItems.Count && itemCount<appointmentsToCache; i++) 
        {
            var item = futureCalItems.Item(i);		
            if (isNull(item))
            {
                item = null;
                break;
            }
            
           var cachedAppointment = processAppointment(item);
            if (cachedAppointment != null)
            {
                itemCount++;
                appointments[itemCount] = cachedAppointment;
            }
            
            item = null;
        }
        
        numAppointments = itemCount;
        futureCalItems = null;
        
        items = null;
        folder = null;
        session = null;
        application = null;
        
        lastUpdateDate = new Date();
    }
    catch (err)
    {
        dbgOut("Failure during update: " + err.description);
    }
    finally
    {
        DisconnectFromOutlook();
        CollectGarbage();
        setDataRefreshTimer(REFRESH_DATA_RATE);
        updateDisplay();
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Connect to Outlook and retrieve upcoming appointments
// UsesOutlook: True
//
////////////////////////////////////////////////////////////////////////////////
function processAppointment(apptItem)
{
	// Ignore all day events
	if ( (apptItem.IsConflict == false) && (apptItem.AllDayEvent == true))
	{
            apptItem = null;
            return null;
	}

    // Ignore appointments than span 24 hours or more
    var startDate = new Date(apptItem.Start);
    if (startDate.setDate(startDate.getDate() + 1) <= new Date(apptItem.End))
    {
        apptItem = null;
        return null;
    }

    return initAppointmentData(apptItem);
}

////////////////////////////////////////////////////////////////////////////////
//
// Create AppointmentData object from Outlook OM item
// UsesOutlook: True
//
////////////////////////////////////////////////////////////////////////////////
function initAppointmentData(item)
{
    //System.Debug.outputString(item.Subject);
    var appt = new AppointmentData();
    appt.Subject = item.Subject;
    var dStart = new Date(item.Start);
    var dEnd = new Date(item.End);
    
    appt.Valid = true;
    try
    {
        appt.InvalidAt = new Date(item.End);
        appt.DateString = bestFitDateTime(dStart, dEnd);
        appt.Location = new String(item.Location);	    
        appt.IsPrivate = (item.Sensitivity == 2);
        appt.BusyState = new Number(item.BusyStatus);
        appt.IsConflict = item.IsConflict;
        
        if (item.IsConflict == false)
        {
            appt.IsRecurring = (item.RecurrenceState != 0);
            appt.EntryID = new String(item.EntryID);
        
            if (propShowOrganizer == "true")
            {
                if (item.MeetingStatus != 0)
                {
                    appt.Organizer = item.Organizer;
                }
                else
                {
                    appt.Organizer = "";
                }
            }
    
            // Get category colors
            var colors = getCategoryColors(item);
            appt.Colors = colors;
        }
        else
        {
            appt.Colors = getCategoryColors(null);
        }
    }
    catch (err)
    {
        dbgOut("Error initalizing appointment: " + err.description);
    }
    
    item = null;
    return appt;
}

////////////////////////////////////////////////////////////////////////////////
//
// Determine if an object is null
//
////////////////////////////////////////////////////////////////////////////////
function isNull(obj)
{
    return ((typeof obj == 'object') && !obj);
}

////////////////////////////////////////////////////////////////////////////////
//
// Rebuild the HTML representation of the appointments
//
////////////////////////////////////////////////////////////////////////////////
function updateDisplay() 
{

    dbgOut("updateDisplay()");
    clearUpdateDisplayTimer();

    if (System.Gadget.visible == false)
    {
        setUpdateDisplayTimer(REFRESH_UI_RATE);
        return;
    }

    var now = new Date();
    // Invalidate appointments (if necessary)
    for (var i=1; i<numAppointments; i++)
    {
        var appt = appointments[i];
        try
        {
			if ((appt.Valid == true) && appt.InvalidAt < now)
			{
				appt.Valid = false;
			}
		}
		catch (ex)
		{
			dbgOut("Problem checking validity.");
		}
    }
    
    // Clear the displayed appointments
    //divAppointments.innerHTML = "";
    
    var appointmentsToRender = 3;
    if (propGadgetSize == "large")
    {
        appointmentsToRender = 5;
    }

    var validAppointmentCount = 0;
    for (var i=1; (i<=numAppointments) && (validAppointmentCount < appointmentsToRender); i++)
    {
        var appt = appointments[i];
        if (appt.Valid == true)
        {
           var divOldAppointment = null;
           dbgOut("apptIndex = " + i);
           if (divAppointments.childNodes.length >= validAppointmentCount + 1)
           {
                divOldAppointment = divAppointments.childNodes.item(validAppointmentCount);
           }
           var divNewAppointment = addAppointment(i, appt);
           if (divOldAppointment != null)
           {
               dbgOut("Found 'old' appointment to replace");
               divAppointments.replaceChild(divNewAppointment, divOldAppointment);
           }
           else
           {
                dbgOut("Creating new appointment");
                divAppointments.appendChild(divNewAppointment);
           }
           validAppointmentCount++;
        }
    }
    if (validAppointmentCount < appointmentsToRender)
    {
        for(var i=validAppointmentCount+1; i<=divAppointments.childNodes.length; i++)
        {
            dbgOut("Removing node " + i + " of " + divAppointments.childNodes.length);
            divAppointments.childNodes.item(i-1).removeNode(true);
        }
    }
    
    setNoData(false, null, null);
    if (isRefreshing)
    {
        setNoData(true, L_RefreshingData_Message);
        waitSpinner.show();
        waitSpinner.start();
    }
    else
    {
        try 
        {
            if (numAppointments == 0 && outlookFailed == true)
            {
                dbgOut("No data to display");
                setNoData(true, L_NoData_Message, true);
            }
            else if (numAppointments == 0)
            {
                dbgOut("No appointments");
                setNoData(true, L_NoUpcomingAppointments_Message, false);
            }
            else if (outlookFailed == true)
            {
                StatusLineText.innerText = L_LastUpdated_StatusBar + dateFormatter.dateToBestFitString(lastUpdateDate);
            }
            else
            {
                StatusLineText.innerText = L_UpToDate_Text;
            }
        }
        catch (err)
        {}
    }
    
    CollectGarbage();
    
    setUpdateDisplayTimer(REFRESH_UI_RATE);
}

////////////////////////////////////////////////////////////////////////////////
//
// Ensure that the timer is set to update the display
//
////////////////////////////////////////////////////////////////////////////////
function setUpdateDisplayTimer()
{
    gadgetRefreshTimer = window.setTimeout("updateDisplay()", REFRESH_UI_RATE);
}

////////////////////////////////////////////////////////////////////////////////
//
// Cancel the timer used to update the display
//
////////////////////////////////////////////////////////////////////////////////
function clearUpdateDisplayTimer()
{
    window.clearTimeout(gadgetRefreshTimer);
}

////////////////////////////////////////////////////////////////////////////////
//
// Create an HTML representation of a single appointment item and return the DIV object
//
////////////////////////////////////////////////////////////////////////////////
function addAppointment(id, appt)
{
    var divAppointment = document.createElement("<div class=\"Appointment\">");
    divAppointment.id = "appt" + id.toString();
    divAppointment.entryId = appt.EntryID;
    divAppointment.ondblclick = showAppt;
    
    var divAppointmentText = document.createElement("<div class=\"AppointmentText\">");
    divAppointment.appendChild(divAppointmentText);
    
    var divSubject = document.createElement("<div class=\"AppointmentSubject\">");
    var subjectLine = L_PrivateAppointmentSubject_Text;
    if (propShowPrivateAppts != "false" || appt.IsPrivate == false)
    {
        subjectLine = appt.Subject;
    }
    var aSubject = document.createElement("a");
    aSubject.innerText = subjectLine;
    aSubject.href = "javascript:void(0)";
    aSubject.onkeypress = appointmentKeyPress;
    
    divSubject.appendChild(aSubject);
    divAppointmentText.appendChild(divSubject);
    
    var divDate = document.createElement("<div class=\"AppointmentDate\">");
    divDate.innerText = appt.DateString;
    divAppointmentText.appendChild(divDate);
    
    var divIcons = document.createElement("<div class=\"IconLine\">");
    
    // Set the icon status	
    if (appt.IsRecurring == true)
    {
        var imgRecurringIcon = document.createElement("<img style=\"width: 10px; height: 11px;\">");
        imgRecurringIcon.src = "images/RecurringAppt" + appt.Colors[1] + ".png";
        divIcons.appendChild(imgRecurringIcon);
    }

    if (appt.IsPrivate == true)
    {
        var imgPrivateIcon = document.createElement("<img style=\"width: 9px; height: 12px;\">");
        imgPrivateIcon.src = "images/PrivateAppt" + appt.Colors[1] + ".png";
        divIcons.appendChild(imgPrivateIcon);
    }
        
    if (appt.IsConflict == true)
    {
        var imgConflictIcon = document.createElement("<img style=\"width: 12px; height: 12px;\">");
        imgConflictIcon.src = "images/ConflictAppt.png";
        divIcons.appendChild(imgConflictIcon);
    }

    divAppointmentText.appendChild(divIcons);
    
    var divLocation = document.createElement("<div class=\"AppointmentLocation\">");
    if (propShowPrivateAppts != "false" || appt.IsPrivate == false)
    {
        if (propShowOrganizer == "true" && appt.Location.length > 0 && appt.Organizer.length>0)
        {
            divLocation.innerText = appt.Location + "; "  + appt.Organizer;
        }
        else if (propShowOrganizer == "true" && appt.Organizer.length>0)
        {
            divLocation.innerText = appt.Organizer;
        }
        else
        {
            divLocation.innerText = appt.Location;
        }
    }
    else
    {
        divLocation.innerText = "";
    }
    divAppointmentText.appendChild(divLocation);
    
    var colors = appt.Colors;
    divAppointment.style.backgroundPositionY = colors[0];
    
    divAppointment.style.color = colors[1];
    aSubject.style.color = colors[1];
    
    var size = "docked";
    if (System.Gadget.docked == false)
    {
        size = "undocked";
    }
        
    switch(appt.BusyState.toString())
    {
        case "0":
            divAppointment.style.backgroundImage = "url(images/appt_free_" + size + ".png)";
            divAppointmentText.style.marginLeft = "8px";
            break;
        case "1":
            divAppointment.style.backgroundImage = "url(images/appt_tentative_" + size + ".png)";
            divAppointmentText.style.marginLeft = "8px";
            break;
        case "3":
            divAppointment.style.backgroundImage = "url(images/appt_oof_" + size + ".png)";
            divAppointmentText.style.marginLeft = "8px";
            break;
        default:
            divAppointment.style.backgroundImage = "url(images/appt_busy_" + size + ".png)";
            divAppointmentText.style.marginLeft = "4px";
            break;
    }
    
    //divAppointments.appendChild(divAppointment);
    return divAppointment;
}

////////////////////////////////////////////////////////////////////////////////
//
// Handle key presses from appointment focus regions
//
////////////////////////////////////////////////////////////////////////////////
function appointmentKeyPress()
{
    if (event.keyCode == 13)
    {
        var element = event.srcElement;
        while (element.id == "")
        {
            element = element.parentNode;
        }
        var id = element.id;
        showAppointmentFromId(id);
    }
}


////////////////////////////////////////////////////////////////////////////////
//
// Event handler for various events that can launch an appointment
//
////////////////////////////////////////////////////////////////////////////////
function showAppt()
{
    var element = event.srcElement;
    while (element.id == "")
    {
        element = element.parentNode;
    }
    var id = element.id;
    showAppointmentFromId(id);
}

////////////////////////////////////////////////////////////////////////////////
//
// Display a particular appointment, based on an index into the appointment array
// UsesOutlook: True
//
////////////////////////////////////////////////////////////////////////////////
function showAppointmentFromId(id)
{
    var apptId = new Number(id.substr(4));
    var entryID = appointments[apptId].EntryID;

    if (ConnectToOutlook()) 
    {
        var session = outlookApplication.Session;
        var item = session.GetItemFromId(entryID);
        item.Display();
        item = null;
        session = null;
        
        DisconnectFromOutlook();
        CollectGarbage();
    }
}


////////////////////////////////////////////////////////////////////////////////
//
// Generate the 'best fit' date string for the appointment
//
////////////////////////////////////////////////////////////////////////////////
function bestFitDateTime(dStart, dEnd) 
{
	return dateFormatter.rangeToBestFitString(dStart, dEnd);
}


////////////////////////////////////////////////////////////////////////////////
//
// Computes if date are on the same day
//
////////////////////////////////////////////////////////////////////////////////
function isDaySame(date1, date2)
{
    try
    {
        // Checks to see if date1 and date2 occur on the same day, with different times
        var d1value = date1.getMonth() + "/" + date1.getDate() + "/" + date1.getFullYear();
        var d2value = date2.getMonth() + "/" + date2.getDate() + "/" + date2.getFullYear();

        return (d1value == d2value);
    }
    catch (err)
    {
        dbgOut("Problem: " + err.description);
    }
}


////////////////////////////////////////////////////////////////////////////////
//
// Returns array of data representing color values for the appointment item
// UsesOutlook: True
//
////////////////////////////////////////////////////////////////////////////////
function getCategoryColors(item)
{
    var defaultOutput = new Object();
    var categoryOffset = 1; // Amount we have to offset to get to the category bubbles
    switch (themeName)
    {
        case "gray":
            defaultOutput[0] = "-47px";
            break;
        default:
            defaultOutput[0] = "0px";
            break;
    }
    defaultOutput[1] = "black";

    if (item == null)
    {
        return defaultOutput;
    }

    try 
    {
        var categories = new String(item.Categories);
        if (categories.length > 0) 
        {
            var category = categories.split(",")[0].replace(/^\s*|\s*$/g,"");
            
            // Look up the category colors
            var olCategory = outlookApplication.Session.Categories.Item(category);
            var output = new Object();
            var numOutput = new Number(olCategory.Color);
            output[0] = ( (numOutput + categoryOffset) * -47) + "px";
            if (numOutput > 11 && numOutput != 13)
            {
                output[1] = "white";
            }
            else
            {
                output[1] = "black";
            }
            return output;
        }
        else
        {
            return defaultOutput;
        }
    }
    catch (err)
    {
        dbgOut("Error; Using default color set.");
        dbgOut(err.description);
        return defaultOutput;
    }
    finally
    {
        item = null;
    }
}
////////////////////////////////////////////////////////////////////////////////
//
// getSpinner( string spinnerDivID ) - establishes a Spinner Object.
//
////////////////////////////////////////////////////////////////////////////////
function getSpinner( spinnerDivID ) 
{
    dbgOut("getSpinner");
    var self = this;
    this.id = spinnerDivID;
    this.parentDiv = document.getElementById( self.id );
    this.fps = 1000/30;  
    with ( this.parentDiv.style ) 
    {
        fontSize=0;
        posWidth=16;
        posHeight=16;
        backgroundImage = 'url("images/activity16v.png")';
        backgroundPositionY = 0;
    }
////////////////////////////////////////////////////////////////////////////////
    this.hide  = function() 
    { 
        self.parentDiv.style.display='none';  
    }
////////////////////////////////////////////////////////////////////////////////
    this.show  = function() 
    { 
        self.parentDiv.style.display='block'; 
    }
////////////////////////////////////////////////////////////////////////////////
    this.stop  = function() 
    { 
        clearInterval( self.animationInterval ); 
    }
////////////////////////////////////////////////////////////////////////////////
    this.start = function() 
    {
        clearInterval( self.animationInterval );
        this.animationInterval = setInterval( 'animateSpinner(' + self.id + ')', 30 );
    }
} 
////////////////////////////////////////////////////////////////////////////////
//
// animateSpinner( oHTMLElement spinnerDiv ) - animates spinner image
//
////////////////////////////////////////////////////////////////////////////////
function animateSpinner(spinnerDiv) 
{
    spinnerDiv.style.backgroundPositionY=parseInt(spinnerDiv.style.backgroundPositionY)-16; 
}

////////////////////////////////////////////////////////////////////////////////
//
// Method used to output debug messages
//
////////////////////////////////////////////////////////////////////////////////
function dbgOut(message)
{
	if (DEBUG)
	{
		System.Debug.outputString(message);
	}
}