////////////////////////////////////////////////////////////////////////////////
//
//  THIS CODE IS NOT REDISTRIBUTABLE OR APPROVED FOR USE IN/ON ANY OTHER UI 
//  ELEMENT OR PRODUCT COMPONENT.
//  Copyright (c) Microsoft Corporation.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
function CustomDateFormatter(daysOfWeekList)
{
    var sTime;          // Time separator
    var sTimeFormat;    // Format string for the Time values
    var s1159;          // Designator for AM
    var s2359;          // Designator for PM
    var iTime;          // 0 = 12 hour clock, 1 = 24 hour clock
    var iTimePrefix;    // location of time time marker (AM or PM), 0 = Suffix, 1 = Prefix
    var iDate;          // General date format (0 = mm/dd/yy, 1 = dd/mm/yy, 2 = yy/mm/dd)
    var sDate;          // Date separator
    var sShortDate;     // Short date format

    var STR_WeekDayNames = daysOfWeekList.split(",");

    var oReg;       // WMI interface, should only be initalized during loadRegionalSettings scope
   
    ////////////////////////////////////////////////////////////////////////////////
    // Private Functions
    function loadRegionalSettings()
    {
        var strComputer = ".";
        oReg = GetObject("winmgmts:{impersonationLevel=impersonate}!\\\\" + strComputer + "\\root\\default:StdRegProv");

        sTime = readRegistryKey("Control Panel\\International", "sTime");
        sTimeFormat = readRegistryKey("Control Panel\\International", "sTimeFormat");
        s1159 = readRegistryKey("Control Panel\\International", "s1159");        
        s2359 = readRegistryKey("Control Panel\\International", "s2359");
        iTime = new Number(readRegistryKey("Control Panel\\International", "iTime"));
        iDate = new Number(readRegistryKey("Control Panel\\International", "iDate"));
        sDate = readRegistryKey("Control Panel\\International", "sDate");
        sShortDate = readRegistryKey("Control Panel\\International", "sShortDate");

        oReg = null;    
        
/*
        sTime = ReadRegKeyString("Control Panel\\International", "sTime");
        sTimeFormat = ReadRegKeyString("Control Panel\\International", "sTimeFormat");
        s1159 = ReadRegKeyString("Control Panel\\International", "s1159");        
        s2359 = ReadRegKeyString("Control Panel\\International", "s2359");
        iTime = new Number(ReadRegKeyString("Control Panel\\International", "iTime"));
        iDate = new Number(ReadRegKeyString("Control Panel\\International", "iDate"));
        sDate = ReadRegKeyString("Control Panel\\International", "sDate");
        sShortDate = ReadRegKeyString("Control Panel\\International", "sShortDate");
*/



        clipTimeFormat();
    }

    ///// Uses WMI to read a value from the regsitry
    function readRegistryKey(strKeyPath, strValueName)
    {
        var HKEY_CURRENT_USER = 2147483649;
        var strValue, dwResult;
        var inParams = oReg.Methods_("GetStringValue").InParameters.SpawnInstance_();
        inParams.hDefKey = HKEY_CURRENT_USER;
        inParams.sSubKeyName = strKeyPath;
        inParams.sValueName = strValueName;
        var outParams = oReg.ExecMethod_("GetStringValue", inParams);
        return outParams.sValue;
    }

    ///// Removes seconds from the time formatting
    function clipTimeFormat()
    {
        // look for and remove seconds from the time format
        var seconds = sTime + "ss";
        var location = sTimeFormat.indexOf(seconds);
        
        var temp = sTimeFormat.substring(0, location) + sTimeFormat.substring(location + 3);
        dbgOut(temp);
        
        sTimeFormat = temp;
    }

    ///// Checks to see if two dates occur on the same day
    function isDaySame(date1, date2)
    {
        try
        {
            // Checks to see if date1 and date2 occur on the same day, with different times
            var d1value = date1.getMonth() + "/" + date1.getDate() + "/" + date1.getFullYear();
            var d2value = date2.getMonth() + "/" + date2.getDate() + "/" + date2.getFullYear();

            return (d1value == d2value);
        }
        catch (err)
        {
            dbgOut("Problem: " + err.description);
        }
    }

    ///// Checks to see if two dates occur in a span of 7 days
    function isWeekSame(date1, date2)
    {
        var d = new Date();
        var date3 = date1;
        date3.setDate(date3.getDate() + 6);
        return (date3 >= date2);
    }
    
    ///// Output the time associated with a date/time variable in the locale setting
    function printTimeOnly(date1)
    {
        var hours = date1.getHours();
        var minutes = date1.getMinutes();
        var timePrefix = s1159;
        if (hours > 11)
        {
            timePrefix = s2359;
        }
        
        if (hours > 12 && iTime == 0)
        {
            hours = hours - 12;
        }
        if (hours == 0 && iTime == 0)
        {
            hours = 12;
        }
        
        var paddedMinutes = new String(minutes);
        if (paddedMinutes.length < 2)
        {
            paddedMinutes = "0" + minutes;
        }
        
        var output = sTimeFormat;
        output = output.replace(/hh/i, hours);
        output = output.replace(/h/i, hours);
        output = output.replace(/mm/, paddedMinutes);
        output = output.replace(/m/, paddedMinutes);
        output = output.replace(/tt/, timePrefix);

        return output;
    }

    ///// Prints out the day of the week for a particular date
    function printWeekDay(date1)
    {
        var dayOfWeek = date1.getDay();
        return STR_WeekDayNames[dayOfWeek];
    }

    ///// Prints the full date information for a particular date
    function printFullDate(date1)
    {
        var output = printWeekDay(date1);
        var month = date1.getMonth() + 1;
        var day = date1.getDate();
        
        var dateString;
        if (iDate == 0 || iDate == 2)
        {
            dateString = month + sDate + day;
        } else {
            dateString = day + sDate + month;
        }
        
        return output + " " + dateString;
    }

    ///// Prints the full date information with a year for a particular date
    function printFullDateWithYear(date1)
    {
        var output = printWeekDay(date1);
        var month = date1.getMonth() + 1;
        var day = date1.getDate();
        var year = date1.getFullYear();
        
        var dateString;
        if (iDate == 0) {
            dateString = month + sDate + day + sDate + year;
        } else if (iDate == 1) {
            dateString = day + sDate + month + sDate + year;
        } else if (iDate == 2) {
            dateString = year + sDate + month + sDate + day;
        }
        
        return output + " " + dateString;
    }

    ///// Used to print debug messages
    function dbgOut(message)
    {
            //System.Debug.outputString(message);
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Public Functions
    this.rangeToBestFitString = function(dateStart, dateEnd)
    {
        var today = new Date();
        var result;
        
        // Figure out the first part of the date
        if (isDaySame(today, dateStart))
        {
            result = printTimeOnly(dateStart)
        }
        else if (isWeekSame(today, dateStart))
        {
            result = printWeekDay(dateStart) + " " + printTimeOnly(dateStart);
        }
        else
        {
            result = printFullDate(dateStart) + " " + printTimeOnly(dateStart);
        }
        
        // Add the end time of the range
        if (isDaySame(dateStart, dateEnd))
        {
            result = result + " - " + printTimeOnly(dateEnd);
        }
        else
        {
            result = result + " - " + printWeekDay(dateEnd) + " " + printTimeOnly(dateEnd);
        }

        
        return result;
    }
    
    this.dateToDisplayString = function(date1)
    {
        return printFullDateWithYear(date1);
    }
    
    this.dateToBestFitString = function(date1)
    {
		var today = new Date();
        var result;
        
        // Figure out the first part of the date
        if (isDaySame(today, date1))
        {
            result = printTimeOnly(date1)
        }
        else if (isWeekSame(today, date1))
        {
            result = printWeekDay(date1) + " " + printTimeOnly(date1);
        }
        else
        {
            result = printFullDate(date1) + " " + printTimeOnly(date1);
        }
        
        return result;
    }
    
    this.toOutlookFilterString = function(date1)
    {
        // We need to use a locale specific date format, but always use 12 hour time formatting
        // WHY: Outlook parses dates using the current locale settings, but "falls back" to US
        // if it doesn't understand the format.  There is no way to express a date in a consistent
        // way that Outlook will parse so we have to use the locale, but work in US format always
        
        // We can't use the built in formatting because Outlook won't accept a date/time string 
        // that includes seconds, and there is no easy way to pull seconds out of the extracted
        // value generated by built-in methods.
        
        var month = date1.getMonth() + 1;
        var day = date1.getDate();
        var year = date1.getFullYear();
        
        
        var dateFormat = sShortDate;
        dateFormat = dateFormat.replace(/MM/, month);
        dateFormat = dateFormat.replace(/M/, month);
        dateFormat = dateFormat.replace(/dd/, day);
        dateFormat = dateFormat.replace(/d/, day);
        dateFormat = dateFormat.replace(/yyyy/, year);
        dateFormat = dateFormat.replace(/yy/, year);
        
        var hours = date1.getHours();
        var minutes = date1.getMinutes();
        var timePrefix = "AM";
        if (hours > 11)
        {
            timePrefix = "PM";
        }
        
        if (hours > 12)
        {
            hours = hours - 12;
            if (hours == 0)
            {
                hours = 12;
            }
        }
        
        var paddedMinutes = new String(minutes);
        if (paddedMinutes.length < 2)
        {
            paddedMinutes = "0" + minutes;
        }
        
        var timeFormat = hours + ":" + paddedMinutes + " " + timePrefix;
        
        
        return dateFormat + " " + timeFormat;
        
    }
    
    this.reload = function()
    {
        loadRegionalSettings();
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Initalize
    loadRegionalSettings();
}

/*
function PartialTestSuite()
{
    WScript.Echo("Running test suite");
    var formatter = new CustomDateFormatter("Sun,Mon,Tue,Wed,Thu,Fri,Sat");
    var sd1 = new Date();
        sd1.setHours(0, 0);
    var ed1 = new Date();
        ed1.setHours(14, 0);
        
    var sd2 = new Date();
        sd2.setHours(13, 01, 00, 00);
        sd2.setFullYear(2006, 11, 7);
    var ed2= new Date();
        ed2.setHours(14, 00, 00, 00);
        ed2.setFullYear(2006, 11, 7);
        
    var sd3 = new Date();
        sd3.setHours(17, 35, 00, 00);
        sd3.setFullYear(2006, 11, 20);
    var ed3= new Date();
        ed3.setHours(23, 00, 00, 00);
        ed3.setFullYear(2006, 11, 20);    
    var value = formatter.rangeToBestFitString(sd1, ed1);
    WScript.Echo("best fit: " + value);
    WScript.Echo("filter string: " + formatter.toOutlookFilterString(sd1));
    WScript.Echo("display string: " + formatter.dateToDisplayString(sd1));
    WScript.Echo("best fit string: " + formatter.dateToBestFitString(sd1));
    value = formatter.rangeToBestFitString(sd2, ed2);
    WScript.Echo("best fit: " + value);
    WScript.Echo("filter string: " + formatter.toOutlookFilterString(sd2));
    WScript.Echo("display string: " + formatter.dateToDisplayString(sd2));
    value = formatter.rangeToBestFitString(sd3, ed3);
    WScript.Echo("best fit: " + value);
    WScript.Echo("filter string: " + formatter.toOutlookFilterString(sd3));
    WScript.Echo("display string: " + formatter.dateToDisplayString(sd3));
}

TestSuite();
*/
